package com.tzld.longarticle.recommend.server.service;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.mapper.crawler.CrawlerBaseMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.net.URLDecoder;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.stream.Collectors;

@Service
@Slf4j
public class DataFlushService {

    @Autowired
    private LongArticleBaseMapper longArticleBaseMapper;
    @Autowired
    private CrawlerBaseMapper crawlerBaseMapper;

    private final ExecutorService pool = ThreadPoolFactory.deDuplicatePool();


    public void flushGetOffVideos(Integer pageNum) {
        int pageSize = 1000;
        if (pageNum == null) {
            pageNum = 1;
        }
        int count = crawlerBaseMapper.countGetOffVideos();
        int totalPage = count / pageSize + 1;
        while (pageNum <= totalPage) {
            int offset = (pageNum - 1) * pageSize;
            List<GetOffVideos> list = crawlerBaseMapper.pageGetOffVideos(offset, pageSize);
            List<Long> videoIds = list.stream().map(GetOffVideos::getVideoId)
                    .distinct().collect(Collectors.toList());
            List<GetOffVideos> existsList = longArticleBaseMapper.getGetOffVideos(videoIds);
            Set<Long> existsIds = existsList.stream().map(GetOffVideos::getVideoId).collect(Collectors.toSet());
            list = list.stream().filter(o -> !existsIds.contains(o.getVideoId())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(list)) {
                longArticleBaseMapper.batchInsertGetOffVideos(list);
            }
            log.info("flushGetOffVideos pageNum:{} totalPage:{}", pageNum, totalPage);
            pageNum++;
        }
    }

    public void flushLongArticlesRootSourceId(Integer pageNum) {
        int pageSize = 1000;
        if (pageNum == null) {
            pageNum = 1;
        }
        int count = crawlerBaseMapper.countLongArticlesRootSourceId();
        int totalPage = count / pageSize + 1;
        while (pageNum <= totalPage) {
            int offset = (pageNum - 1) * pageSize;
            List<LongArticlesRootSourceId> list = crawlerBaseMapper.pageLongArticlesRootSourceId(offset, pageSize);
            List<String> rootSourceIdList = list.stream().map(LongArticlesRootSourceId::getRootSourceId)
                    .distinct().collect(Collectors.toList());
            List<LongArticlesRootSourceId> existsList = longArticleBaseMapper.getLongArticlesRootSourceId(rootSourceIdList);
            Set<String> existsIds = existsList.stream().map(LongArticlesRootSourceId::getRootSourceId).collect(Collectors.toSet());
            list = list.stream().filter(o -> !existsIds.contains(o.getRootSourceId())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(list)) {
                longArticleBaseMapper.batchInsertLongArticlesRootSourceId(list);
            }
            log.info("flushLongArticlesRootSourceId pageNum:{} totalPage:{} existsSize: {}", pageNum, totalPage, existsList.size());
            pageNum++;
        }
    }

    public void flushLongArticlesCrawlerVideos(Integer pageNum) {
        List<ArticleMatchVideos> list = crawlerBaseMapper.pageArticleMatchVideos();
        list = list.stream().filter(o -> StringUtils.hasText(o.getVideoPath())).collect(Collectors.toList());
        Map<String, List<ArticleMatchVideos>> map = list.stream().collect(Collectors.groupingBy(ArticleMatchVideos::getContentId));
        List<LongArticlesCrawlerVideos> batchSaveList = new ArrayList<>();
        for (ArticleMatchVideos articleMatchVideos : list) {
            List<ArticleMatchVideos> mapList = map.get(articleMatchVideos.getContentId());
            List<Date> orderDate = mapList.stream().map(ArticleMatchVideos::getUpdateTime)
                    .sorted().collect(Collectors.toList());
            double score = 0.2;
            for (int i = 0; i < orderDate.size(); i++) {
                if (orderDate.get(i).equals(articleMatchVideos.getUpdateTime())) {
                    if (i == 0) {
                        score = 1;
                    } else if (i == 1) {
                        score = 0.5;
                    } else {
                        break;
                    }
                }
            }
            LongArticlesCrawlerVideos saveItem = new LongArticlesCrawlerVideos();
            saveItem.setContentId(articleMatchVideos.getContentId());
            saveItem.setPlatform(articleMatchVideos.getPlatform());
            saveItem.setVideoTitle(articleMatchVideos.getVideoTitle());
            saveItem.setCrawlerTime(articleMatchVideos.getUpdateTime());
            saveItem.setVideoOssPath(articleMatchVideos.getVideoPath());
            saveItem.setCoverOssPath(articleMatchVideos.getCoverPath());
            saveItem.setUserId(articleMatchVideos.getUid());
            saveItem.setTraceId(articleMatchVideos.getTraceId());
            saveItem.setDownloadStatus(2);
            saveItem.setScore(score / 1000);
            batchSaveList.add(saveItem);
        }
        if (!CollectionUtils.isEmpty(batchSaveList)) {
            for (List<LongArticlesCrawlerVideos> partition : Lists.partition(batchSaveList, 1000)) {
                List<String> contentIds = partition.stream().map(LongArticlesCrawlerVideos::getContentId).collect(Collectors.toList());
                List<LongArticlesCrawlerVideos> existsList = longArticleBaseMapper.getLongArticlesCrawlerVideos(contentIds);
                Map<String, List<LongArticlesCrawlerVideos>> existsMap = existsList.stream()
                        .collect(Collectors.groupingBy(LongArticlesCrawlerVideos::getContentId));
                partition = partition.stream().filter(o -> {
                    List<LongArticlesCrawlerVideos> itemList = existsMap.get(o.getContentId());
                    return CollectionUtil.isEmpty(itemList) || itemList.size() < 3;
                }).collect(Collectors.toList());
                longArticleBaseMapper.batchInsertLongArticlesCrawlerVideos(partition);
            }
        }
    }

    public void flushLongArticlesVideos(Integer pageNum) {
        int pageSize = 1000;
        if (pageNum == null) {
            pageNum = 1;
        }
        List<LongArticlesText> kimiTitleList = crawlerBaseMapper.getLongArticlesText();
        Map<String, LongArticlesText> kimiTitleMap = kimiTitleList.stream().collect(
                Collectors.toMap(LongArticlesText::getContentId, o -> o, (existing, replacement) -> replacement));
        int count = crawlerBaseMapper.countLongArticlesVideos();
        int totalPage = count / pageSize + 1;
        while (pageNum <= totalPage) {
            int offset = (pageNum - 1) * pageSize;
            List<LongArticlesVideo> list = crawlerBaseMapper.pageLongArticlesVideos(offset, pageSize);
            List<LongArticlesText> batchSaveLongArticlesTextList = new ArrayList<>();
            List<LongArticlesMatchVideos> batchSaveLongArticlesMatchVideosList = new ArrayList<>();
            Set<String> existsIdSet = new HashSet<>();
            for (LongArticlesVideo longArticlesVideo : list) {
                if (longArticlesVideo.getContentId().endsWith("lehuo")) {
                    continue;
                }
                if (!existsIdSet.contains(longArticlesVideo.getContentId())) {
                    LongArticlesText longArticlesText = new LongArticlesText();
                    longArticlesText.setContentId(longArticlesVideo.getContentId());
                    longArticlesText.setArticleTitle(longArticlesVideo.getArticleTitle());
                    longArticlesText.setArticleText(longArticlesVideo.getArticleText());
                    if (StringUtils.hasText(longArticlesVideo.getArticleText())) {
                        longArticlesText.setKimiTitle(longArticlesVideo.getKimiTitle().replace("\"", ""));
                    }
                    if (StringUtils.hasText(longArticlesVideo.getKimiSummary())) {
                        longArticlesText.setKimiSummary(longArticlesVideo.getKimiSummary());
                        longArticlesText.setKimiKeys(longArticlesVideo.getKimiKeys());
                    } else {
                        LongArticlesText text = kimiTitleMap.get(longArticlesVideo.getContentId());
                        longArticlesText.setKimiSummary(text.getKimiSummary());
                        longArticlesText.setKimiKeys(text.getKimiKeys());
                    }
                    longArticlesText.setKimiStatus(1);
                    batchSaveLongArticlesTextList.add(longArticlesText);
                    existsIdSet.add(longArticlesVideo.getContentId());
                }
                if (Objects.isNull(longArticlesVideo.getRequestTimeStamp())) {
                    continue;
                }
                LongArticlesMatchVideos longArticlesMatchVideos = new LongArticlesMatchVideos();
                longArticlesMatchVideos.setContentId(longArticlesVideo.getContentId());
                longArticlesMatchVideos.setTraceId(longArticlesVideo.getTraceId());
                longArticlesMatchVideos.setGhId(longArticlesVideo.getGhId());
                longArticlesMatchVideos.setAccountName(longArticlesVideo.getAccountName());
                longArticlesMatchVideos.setContentStatus(longArticlesVideo.getContentStatus());
                longArticlesMatchVideos.setSuccessStatus(longArticlesVideo.getSuccess());
                longArticlesMatchVideos.setRequestTimestamp(longArticlesVideo.getRequestTimeStamp());
                longArticlesMatchVideos.setUpdateTime(longArticlesVideo.getUpdateTime());
                longArticlesMatchVideos.setProcessTimes(longArticlesVideo.getProcessTimes());
                longArticlesMatchVideos.setResponse(getLongArticleVideoResponse(longArticlesVideo));
                batchSaveLongArticlesMatchVideosList.add(longArticlesMatchVideos);
            }
            if (CollectionUtil.isNotEmpty(batchSaveLongArticlesTextList)) {
                List<String> contentIds = batchSaveLongArticlesTextList.stream()
                        .map(LongArticlesText::getContentId).distinct().collect(Collectors.toList());
                List<String> existsContentIds = longArticleBaseMapper.getLongArticlesTextByContentIds(contentIds);
                if (CollectionUtil.isNotEmpty(existsContentIds)) {
                    batchSaveLongArticlesTextList = batchSaveLongArticlesTextList.stream()
                            .filter(o -> !existsContentIds.contains(o.getContentId())).collect(Collectors.toList());
                }
                if (CollectionUtil.isNotEmpty(batchSaveLongArticlesTextList)) {
                    longArticleBaseMapper.batchInsertLongArticlesText(batchSaveLongArticlesTextList);
                }
            }
            if (CollectionUtil.isNotEmpty(batchSaveLongArticlesMatchVideosList)) {
                List<String> traceIds = batchSaveLongArticlesMatchVideosList.stream().map(LongArticlesMatchVideos::getTraceId)
                        .distinct().collect(Collectors.toList());
                List<LongArticlesMatchVideos> existsList = longArticleBaseMapper.getLongArticlesMatchVideos(traceIds);
                Set<String> existsIds = existsList.stream().map(LongArticlesMatchVideos::getTraceId).collect(Collectors.toSet());
                batchSaveLongArticlesMatchVideosList = batchSaveLongArticlesMatchVideosList.stream()
                        .filter(o -> !existsIds.contains(o.getTraceId())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(batchSaveLongArticlesMatchVideosList)) {
                    longArticleBaseMapper.batchInsertLongArticlesMatchVideos(batchSaveLongArticlesMatchVideosList);
                }
            }
            log.info("flushLongArticlesVideos pageNum:{} totalPage:{}", pageNum, totalPage);
            pageNum++;
        }
    }

    private String getLongArticleVideoResponse(LongArticlesVideo longArticlesVideo) {
        JSONArray jsonArray = new JSONArray();
        if (StringUtils.hasText(longArticlesVideo.getResult1())) {
            if (StringUtils.hasText(longArticlesVideo.getKimiTitle()) && longArticlesVideo.getKimiTitle().contains("\"")) {
                longArticlesVideo.setResult1(longArticlesVideo.getResult1().replace(longArticlesVideo.getKimiTitle(),
                        longArticlesVideo.getKimiTitle().replace("\"", "")));
            }
            jsonArray.add(resultToResponse(longArticlesVideo.getResult1()));
        }
        if (StringUtils.hasText(longArticlesVideo.getResult2())) {
            if (StringUtils.hasText(longArticlesVideo.getKimiTitle()) && longArticlesVideo.getKimiTitle().contains("\"")) {
                longArticlesVideo.setResult2(longArticlesVideo.getResult2().replace(longArticlesVideo.getKimiTitle(),
                        longArticlesVideo.getKimiTitle().replace("\"", "")));
            }
            jsonArray.add(resultToResponse(longArticlesVideo.getResult2()));
        }
        if (StringUtils.hasText(longArticlesVideo.getResult3())) {
            if (StringUtils.hasText(longArticlesVideo.getKimiTitle()) && longArticlesVideo.getKimiTitle().contains("\"")) {
                longArticlesVideo.setResult3(longArticlesVideo.getResult3().replace(longArticlesVideo.getKimiTitle(),
                        longArticlesVideo.getKimiTitle().replace("\"", "")));
            }
            jsonArray.add(resultToResponse(longArticlesVideo.getResult3()));
        }
        return JSONObject.toJSONString(jsonArray);
    }

    private JSONObject resultToResponse(String result) {
        JSONObject jsonObject = new JSONObject();
        JSONObject fromJSON = JSONObject.parseObject(result);
        jsonObject.put("kimiTitle", fromJSON.getString("productionName"));
        jsonObject.put("videoCover", fromJSON.getString("productionCover"));
        jsonObject.put("videoPath", fromJSON.getString("videoUrl"));
        jsonObject.put("source", fromJSON.getString("source"));
        String productionPath = fromJSON.getString("productionPath");
        String uid = getParamFromPath(productionPath, "su");
        String videoId = getParamFromPath(productionPath, "id");
        jsonObject.put("uid", uid);
        if (StringUtils.hasText(videoId)) {
            jsonObject.put("videoId", Long.valueOf(videoId));
        }
        return jsonObject;
    }

    private String getParamFromPath(String productionPath, String param) {
        String decode = URLDecoder.decode(productionPath);
        String[] sss = decode.split("\\?");
        for (String ss : sss) {
            String[] split = ss.split("&");
            for (String s : split) {
                if (s.startsWith(param)) {
                    String[] uid = s.split("=");
                    return uid[1];
                }
            }
        }
        return null;
    }

    public void flushLongArticlesText() {
        List<LongArticlesText> kimiTitleList = crawlerBaseMapper.getLongArticlesText();
        Map<String, LongArticlesText> kimiTitleMap = kimiTitleList.stream().collect(
                Collectors.toMap(LongArticlesText::getContentId, o -> o, (existing, replacement) -> replacement));
        List<LongArticlesText> updateList = longArticleBaseMapper.getNeedUpdateRecords();
        for (LongArticlesText update : updateList) {
            LongArticlesText kimi = kimiTitleMap.get(update.getContentId());
            if (Objects.nonNull(kimi)) {
                pool.submit(() -> {
                    update.setKimiTitle(kimi.getKimiTitle());
                    update.setKimiSummary(kimi.getKimiSummary());
                    update.setKimiKeys(kimi.getKimiKeys());
                    longArticleBaseMapper.updateLongArticlesText(update);
                });
            }
        }
    }
}
